-----------------------------------------------------
-- Beispielskript zur SNEK 5 im Mrz 2016 in Nrnberg
-- Autor: Bernd Jungbluth
-- Keine Gewhrleistung, keine Haftung

-- Beispielskript 01 - Temporal Table aktivieren
-----------------------------------------------------
USE SNEK;
GO

-------------------------------------------------------
-- Anlegen einer Tabelle mit aktivierter Temporal Table
-------------------------------------------------------

-- Tabelle anlegen 
CREATE TABLE dbo.Konferenzen
(
	KonferenzID		int				NOT NULL	
					IDENTITY(1,1)	PRIMARY KEY CLUSTERED,
	Konferenz		nvarchar(50)	NOT NULL,
	Veranstalter	nvarchar(100)	NOT NULL,
	Termin			date,

	-- Notwendige Spalten fr Temporal Table
	AktuellVon		datetime2(0) GENERATED ALWAYS AS ROW START,
	AktuellBis		datetime2(0) GENERATED ALWAYS AS ROW END,
	PERIOD FOR SYSTEM_TIME (AktuellVon, AktuellBis)
)
-- Aktivierung Temporal Table mit Namensvergabe fr Temporal Table
WITH (SYSTEM_VERSIONING = ON (History_Table = dbo.HistorieKonferenzen));

-- Index der Temporal Table lschen
DROP INDEX ix_HistorieKonferenzen ON dbo.HistorieKonferenzen;
-- Temporal Table mit zur Benutzertabelle abweichenden Indexen ergnzen
CREATE CLUSTERED COLUMNSTORE INDEX CCI__HistorieKonferenzen 
	   ON dbo.HistorieKonferenzen;
CREATE NONCLUSTERED INDEX IX__HistorieKonferenzen 
	   ON dbo.HistorieKonferenzen (AktuellVon, AktuellBis, KonferenzID);


-------------------------------------------------------
-- Lschen einer Benutzertabelle mit Temporal Table
-------------------------------------------------------

-- System Versioning ausschalten
ALTER TABLE dbo.Konferenzen SET (SYSTEM_VERSIONING = OFF);
-- Temporal Table lschen
DROP TABLE dbo.HistorieKonferenzen;
-- Benutzertabelle
DROP TABLE dbo.Konferenzen;
GO


-----------------------------------------------------------------------
-- Aktivieren einer Temporal Table an einer bestehenden Benutzertabelle
-----------------------------------------------------------------------

-- Inhalt der Tabelle ausgeben
SELECT * FROM dbo.Artikel;

-- Zunchst die notwendigen Spalten an Benutzertabelle ergnzen
--> Mit Zusatz HIDDEN, um bestehende Abfragen nicht zu gefhrden
ALTER TABLE dbo.Artikel ADD
	AktuellVon datetime2(0)	GENERATED ALWAYS AS ROW START 
							HIDDEN
							CONSTRAINT DF__dbo_Artikel_Systemzeit 
									   DEFAULT SYSUTCDATETIME(),
	AktuellBis datetime2(0)	GENERATED ALWAYS AS ROW END 
							HIDDEN
							CONSTRAINT DF__dbo_Artikel_EndeZeit 
									   DEFAULT (CAST('9999-12-31 23:59:59' As datetime2(0))),
	PERIOD FOR SYSTEM_TIME (AktuellVon, AktuellBis);

-- Inhalt der Tabelle ausgeben
--> neue Spalten werden nicht angezeigt
SELECT * FROM dbo.Artikel;

-- System Versioning fr Tabelle aktivieren
ALTER TABLE dbo.Artikel
	SET (SYSTEM_VERSIONING = ON (HISTORY_TABLE = Historie.Artikel));