--------------------------------------------------------------------
-- Beispielskript zur SNEK 5 im Mrz 2016 in Nrnberg
-- Autor: Bernd Jungbluth
-- Keine Gewhrleistung, keine Haftung

-- Beispielskript 08 - Dynamic Data Masking - Definition und Ausgabe
--------------------------------------------------------------------
USE SNEK;
GO


-- Maskieren der Spalte Geburtsdatum mit Standardwert des Datentyps der Spalte
ALTER TABLE dbo.Personal ALTER COLUMN Geburtsdatum 
		ADD MASKED WITH (FUNCTION = 'default()');


-- Daten als dbo ausgeben
SELECT	Nachname, Vorname, Geburtsdatum 
FROM	dbo.Personal;
--> Geburtsdatum lesbar

-- Daten als Benutzer Bundestrojaner ausgeben
-- Keine Angst, ist nur Spa :-)
EXECUTE AS USER = 'Bundestrojaner';
SELECT	Nachname, Vorname, Geburtsdatum 
FROM	dbo.Personal;
REVERT;
--> Geburtsdatum maskiert


-- Maskierung fr Benutzer aufheben
GRANT UNMASK TO Bundestrojaner;


-- Daten als Benutzer Bundestrojaner ausgeben
EXECUTE AS USER = 'Bundestrojaner';
SELECT	Nachname, Vorname, Geburtsdatum 
FROM	dbo.Personal;
REVERT;


-- Maskierung fr Benutzer Bundestrojaner wieder aktivieren
REVOKE UNMASK TO Bundestrojaner;


-- Daten als Benutzer Bundestrojaner ausgeben
EXECUTE AS USER = 'Bundestrojaner';
SELECT	Nachname, Vorname, Geburtsdatum 
FROM	dbo.Personal;
REVERT;


-- Maskieren der Spalte Menge mit zuflligen Werten aus angegebenen Bereich
ALTER TABLE dbo.Auftragspositionen ALTER COLUMN Menge 
		ADD MASKED WITH (FUNCTION = 'random(20000, 30000)');


-- Daten als Benutzer Bundestrojaner ausgeben
EXECUTE AS USER = 'Bundestrojaner';
SELECT	AuftragID, PosNr, Einzelpreis, Menge
FROM	dbo.Auftragspositionen
WHERE	AuftragID BETWEEN 5580 AND 5585;
REVERT;

-- Daten als dbo ausgeben
SELECT	AuftragID, PosNr, Einzelpreis, Menge
FROM	dbo.Auftragspositionen
WHERE	AuftragID BETWEEN 5580 AND 5585;


-- Maskieren der Spalte Email mit entsprechender Funktion 
ALTER TABLE dbo.Personal ALTER COLUMN Email 
		ADD MASKED WITH (FUNCTION = 'email()');


-- Daten als Benutzer Bundestrojaner ausgeben
EXECUTE AS USER = 'Bundestrojaner';
SELECT	Nachname, Vorname, Email 
FROM	dbo.Personal;
REVERT;


-- Maskieren der Spalte Email mit eigener partieller Maskierung
ALTER TABLE dbo.Personal ALTER COLUMN Email 
		ADD MASKED WITH (FUNCTION = 'partial(2," -_- ", 2)');


-- Daten als Benutzer Bundestrojaner ausgeben
EXECUTE AS USER = 'Bundestrojaner';
SELECT	Nachname, Vorname, Email 
FROM	dbo.Personal;
REVERT;


-- bersicht aller Tabellen mit maskierten Spalten ausgeben
SELECT	object_name(object_id) As Tabelle, 
		name As Spalte, masking_function As Maske
FROM	sys.masked_columns
WHERE	is_masked = 1;


-- Maskierung entfernen
ALTER TABLE dbo.Personal ALTER COLUMN Email DROP MASKED;


-- Daten als Benutzer Bundestrojaner ausgeben
EXECUTE AS USER = 'Bundestrojaner';
SELECT	Nachname, Vorname, Email 
FROM	dbo.Personal;
REVERT;