-----------------------------------------------------
-- Beispielskript zur SNEK 5 im Mrz 2016 in Nrnberg
-- Autor: Bernd Jungbluth
-- Keine Gewhrleistung, keine Haftung

-- Beispielskript 12 - T-SQL - COMPRESS
-----------------------------------------------------
USE SNEK;
GO


-- Beispieltabelle fr BLOB-Werte anlegen
DROP TABLE IF EXISTS dbo.Blobs;
CREATE TABLE dbo.Blobs
(
	BlobID	INT IDENTITY(1,1),
	Blob	NVARCHAR(max)	
)
GO

-- Optimiertes Speichern von Blobs aktivieren
EXEC sp_tableoption 'Blobs', 'Large Value Types out of row', true
GO

-- Daten in Tabelle einfgen
INSERT INTO Blobs (Blob) 
	 VALUES (REPLICATE(N'Dies ist 1000mal der gleiche lange unkomprimierte Text. ', 1000));
GO 500

-- Daten ausgeben
SELECT BlobID, Blob FROM dbo.Blobs;


-- Anzahl der Datenseiten ermitteln
SELECT
		allocation_unit_type_desc, page_type_desc, COUNT(*) As Datapages
FROM	
		sys.dm_db_database_page_allocations (DB_ID('SNEK'), OBJECT_ID('Blobs'), 0, 1, 'DETAILED')
GROUP BY
		allocation_unit_type_desc, page_type_desc
ORDER BY
		allocation_unit_type_desc;
--> 506 Datapages fr Blobs



-- Tabelle leeren
TRUNCATE TABLE dbo.Blobs;


-- Daten in Tabelle einfgen - mit COMPRESS
INSERT INTO Blobs (Blob) 
	 VALUES (COMPRESS(REPLICATE(N'Dies ist 1000mal der gleiche lange komprimierte Text. ', 1000)));
GO 500


-- Anzahl der Datenseiten ermitteln
SELECT
		allocation_unit_type_desc, page_type_desc, COUNT(*) As Datapages
FROM	
		sys.dm_db_database_page_allocations (DB_ID('SNEK'), OBJECT_ID('Blobs'), 0, 1, 'DETAILED')
GROUP BY
		allocation_unit_type_desc, page_type_desc
ORDER BY
		allocation_unit_type_desc;
--> 16 Datenseiten fr Blobs



-- Komprierte Daten ausgeben
SELECT TOP 5 BlobID, Blob FROM dbo.Blobs;

-- Daten entkomprimiert ausgeben 
SELECT TOP 5 DECOMPRESS(Blob) As Blob FROM dbo.Blobs;
--> Fehler

-- Daten in VARBINARY konvertieren und entkomprimiert ausgeben
SELECT TOP 5 DECOMPRESS(CAST(Blob As VARBINARY(max))) As Blob FROM dbo.Blobs;

-- Daten in VARBINARY konvertieren und entkomprimierten Wert in NVARCHAR(MAX) konvertieren und ausgeben
SELECT TOP 5 CAST(DECOMPRESS(CAST(Blob As VARBINARY(max))) As NVARCHAR(max)) As Blob FROM dbo.Blobs;
GO



-- Bessere Variante fr komprimierte Daten
--> Spalte fr Blob direkt als VARBINARY definieren
DROP TABLE IF EXISTS dbo.Blobs;
CREATE TABLE dbo.Blobs
(
	BlobID		INT IDENTITY(1,1),
	KompBlob	VARBINARY(max)		
)
GO

-- Daten in Tabelle einfgen - mit COMPRESS
INSERT INTO Blobs (KompBlob) 
	 VALUES (COMPRESS(REPLICATE(N'Dies ist 1000mal der gleiche langer unkomprimierte Text.', 1000)));

-- Entkomprimierten Wert in NVARCHAR(MAX) konvertieren und ausgeben
SELECT CAST(DECOMPRESS(KompBlob) As NVARCHAR(max)) As KompBlob FROM dbo.Blobs;