-----------------------------------------------------
-- Beispielskript zur SNEK 5 im Mrz 2016 in Nrnberg
-- Autor: Bernd Jungbluth
-- Keine Gewhrleistung, keine Haftung

-- Beispielskript 13 - JSON
-----------------------------------------------------
USE SNEK;
GO



-------------------------------------------------------------
-- Daten der Tabelle Artikelgruppe in JSON-Format exportieren
-------------------------------------------------------------

-- Export per FOR JSON
SELECT	ArtikelgruppeID, Artikelgruppe, AgBeschreibung 
FROM	dbo.Artikelgruppen
FOR		JSON PATH, ROOT('Artikelgruppen')
GO



--------------------------------------------------------------------------
-- Daten im JSON-Format als Datensatz in Tabelle Artikelgruppe importieren
--------------------------------------------------------------------------

-- JSON als Variable
DECLARE @JsonImport NVARCHAR(max) = 
'{
	"Artikelgruppe": "Damensocke \"Modern\"",
	"AgBeschreibung": "klassische Damensocke; 100% Baumwolle, kurz"
 }';

-- Import per OPENJSON
INSERT INTO dbo.Artikelgruppen (Artikelgruppe, AgBeschreibung)
SELECT	Artikelgruppe, AgBeschreibung
FROM	OPENJSON (@JsonImport)
		WITH (Artikelgruppe nvarchar(50),
			  AgBeschreibung nvarchar(max));

-- Daten der Tabelle Artikelgruppen ausgeben
SELECT	ArtikelgruppeID, Artikelgruppe, AgBeschreibung 
FROM	dbo.Artikelgruppen;
GO



--------------------------------------------------------------------
-- Daten im JSON-Format in Tabelle speichern und per T-SQL auswerten
--------------------------------------------------------------------

-- Tabelle fr JSON-Dokumente anlegen
CREATE TABLE dbo.ArtikelJson
(
	ArtikelJsonID	INT IDENTITY(1,1) PRIMARY KEY,
	InhaltJson		NVARCHAR(max) 
					CONSTRAINT InhaltJSON_ISJSON_1 CHECK (ISJSON(InhaltJSON) > 0)
);
GO

-- Daten in Tabelle einfgen
INSERT INTO dbo.ArtikelJson (InhaltJson)
	VALUES (N'{"Artikelgruppe":"Damensocke \"Klassik\"",
			   "AgBeschreibung":"klassische Damensocke; 100% Baumwolle, glatt",
			   "Farbe":"rot"}');
INSERT INTO dbo.ArtikelJson (InhaltJson)
	VALUES (N'{"Artikelgruppe":"Herrensocke \"Klassik\"",
				"AgBeschreibung":"klassische Herrensocke; 100% Baumwolle, glatt",
				"Farbe":"lila"}');
INSERT INTO dbo.ArtikelJson (InhaltJson)
	VALUES (N'{"Artikelgruppe":"Sportsocke \"Run\"",
				"AgBeschreibung":"klassische Sportsocke; 100% Baumwolle",
				"Farbe":"rot"}');

-- Daten der Tabelle ausgeben
SELECT * FROM dbo.ArtikelJSON;


-- JSON-Daten der Tabelle aufbereiten und ausgeben
SELECT	ArtikelJsonID As ID,
		JSON_VALUE(InhaltJson, '$.Artikelgruppe') As Artikelgruppe,
		JSON_VALUE(InhaltJson, '$.AgBeschreibung') As Beschreibung,
		JSON_VALUE(InhaltJson, '$.Farbe') As Farbe
FROM	dbo.ArtikelJson
WHERE	ISJSON(InhaltJson) > 0
		AND
		JSON_VALUE(InhaltJson, '$.Farbe') = 'rot';


-- Beispieltabelle lschen
DROP TABLE dbo.ArtikelJson;
